@@ -87,7 +87,33 @@ class GeocacheCoordinate(geo.Coordinate):
     }
 
 
-    SQLROW = {'lat': 'REAL', 'lon': 'REAL', 'name': 'TEXT PRIMARY KEY', 'title': 'TEXT', 'shortdesc': 'TEXT', 'desc': 'TEXT', 'hints': 'TEXT', 'type': 'TEXT', 'size': 'INTEGER', 'difficulty': 'INTEGER', 'terrain': 'INTEGER', 'owner': 'TEXT', 'found': 'INTEGER', 'waypoints': 'text', 'images': 'text', 'notes': 'TEXT', 'fieldnotes': 'TEXT', 'logas': 'INTEGER', 'logdate': 'TEXT', 'marked' : 'INTEGER', 'logs' : 'TEXT', 'status' : 'INTEGER', 'vars' : 'TEXT'}
+    SQLROW = {
+        'lat': 'REAL',
+        'lon': 'REAL',
+        'name': 'TEXT PRIMARY KEY',
+        'title': 'TEXT',
+        'shortdesc': 'TEXT',
+        'desc': 'TEXT',
+        'hints': 'TEXT',
+        'type': 'TEXT',
+        'size': 'INTEGER',
+        'difficulty': 'INTEGER',
+        'terrain': 'INTEGER',
+        'owner': 'TEXT',
+        'found': 'INTEGER',
+        'waypoints': 'text',
+        'images': 'text',
+        'notes': 'TEXT',
+        'fieldnotes': 'TEXT',
+        'logas': 'INTEGER',
+        'logdate': 'TEXT',
+        'marked' : 'INTEGER',
+        'logs' : 'TEXT',
+        'status' : 'INTEGER',
+        'vars' : 'TEXT',
+        'alter_lat' : 'REAL',
+        'alter_lon' : 'REAL'
+        }
     def __init__(self, lat, lon, name=''):
         geo.Coordinate.__init__(self, lat, lon, name)
         # NAME = GC-ID
@@ -111,12 +137,15 @@ class GeocacheCoordinate(geo.Coordinate):
         self.logs = ''
         self.status = self.STATUS_NORMAL
         self.vars = ''
+        self.alter_lat = 0
+        self.alter_lon = 0
 
     def clone(self):
         n = GeocacheCoordinate(self.lat, self.lon)
         for k in ('title', 'name', 'shortdesc', 'desc', 'hints', 'type', \
             'size', 'difficulty', 'terrain', 'owner', 'found', 'waypoints', \
-            'images', 'notes', 'fieldnotes', 'log_as', 'log_date', 'marked', 'logs', 'status'):
+            'images', 'notes', 'fieldnotes', 'log_as', 'log_date', 'marked', \
+            'logs', 'status', 'vars', 'alter_lat', 'alter_lon'):
             setattr(n, k, getattr(self, k))
         return n
         
@@ -162,7 +191,9 @@ class GeocacheCoordinate(geo.Coordinate):
             'marked' : marked,
             'logs' : self.logs,
             'status' : self.status,
-            'vars' : self.vars
+            'vars' : self.vars,
+            'alter_lat' : self.alter_lat,
+            'alter_lon' : self.alter_lon,
         }
                 
     def unserialize(self, data):
@@ -201,6 +232,8 @@ class GeocacheCoordinate(geo.Coordinate):
         else:
             self.vars = data['vars']
         self.status = data['status']
+        self.alter_lat = data['alter_lat']
+        self.alter_lon = data['alter_lon']
 
     def get_waypoints(self):
         if self.waypoints == None or self.waypoints == '':
@@ -263,6 +296,10 @@ class GeocacheCoordinate(geo.Coordinate):
         else:
             return self.TYPE_MAPPING[self.TYPE_UNKNOWN]
 
+    def set_alternative_position(self, coord):
+        self.alter_lat = coord.lat
+        self.alter_lon = coord.lon
+
 class FieldnotesUploader(gobject.GObject):
     __gsignals__ = { 'finished-uploading': (gobject.SIGNAL_RUN_FIRST,\
                                  gobject.TYPE_NONE,\
@@ -362,7 +399,7 @@ class CacheDownloader(gobject.GObject):
 
     @staticmethod
     def __replace_br(text):
-        return re.sub('<[bB][rR]\s*/?>|</[pP]>', '\n', text)
+        return re.sub('<[bB][rR]\s*/?>|</?[pP]>', '\n', text)
 
     def __treat_hints(self, hints):
         hints = self.__strip_html(self.__replace_br(hints)).strip()
@@ -416,7 +453,7 @@ class CacheDownloader(gobject.GObject):
                              'lon': self.__from_dm(m.group('lon_sign'), m.group('lon_d'), m.group('lon_m')),
                              'id': "%s%s" % m.group('id_prefix', 'id'),
                              'name': self.__decode_htmlentities(m.group('name')),
-                             'comment': self.__decode_htmlentities(self.__strip_html(self.__replace_br(m.group('comment'))))
+                             'comment': self.__decode_htmlentities(self.__strip_html(self.__replace_br(m.group('comment')), True))
                              })
 
         return waypoints
@@ -553,14 +590,14 @@ class CacheDownloader(gobject.GObject):
         self.images = {}
         coordinate = coordinate.clone()
         self.current_cache = coordinate
-        try:
-            print "* Downloading %s..." % (coordinate.name)
-            response = self.__get_cache_page(coordinate.name)
-            u = self.__parse_cache_page(response, coordinate)
-        except Exception, e:
-            CacheDownloader.lock.release()
-            self.emit('download-error', e)
-            return self.current_cache
+        #try:
+        print "* Downloading %s..." % (coordinate.name)
+        response = self.__get_cache_page(coordinate.name)
+        u = self.__parse_cache_page(response, coordinate)
+        #except Exception, e:
+        #    CacheDownloader.lock.release()
+        #    self.emit('download-error', e)
+        #    return self.current_cache
         CacheDownloader.lock.release()
         self.emit('finished-single', u)
         return u
@@ -644,7 +681,7 @@ class CacheDownloader(gobject.GObject):
         return points
                 
     def __parse_cache_page(self, cache_page, coordinate):
-        indesc = inshortdesc = inwaypoints = False
+        indesc = inshortdesc = inwaypoints = inhints = False
         inhead = True
         shortdesc = desc = hints = waypoints = images = logs = owner = ''
         for line in cache_page:
@@ -662,8 +699,13 @@ class CacheDownloader(gobject.GObject):
                 inhead = False
                 inshortdesc = False
                 indesc = False
-            elif line.startswith('<p><span id="ctl00_ContentBody_Hints" class="displayMe">'):
-                hints = re.compile('<span id="ctl00_ContentBody_Hints"[^>]+>(.*?)</span>').search(line).group(1)
+            elif line.startswith('<div id="div_hint" class="HalfLeft">'):
+                inhead = False
+                inshortdesc = False
+                indesc = False
+                inhints = True
+            elif inhints and line.startswith('</div>'):
+                inhints = False
             elif line.startswith('<div id="ctl00_ContentBody_uxlrgMap" class="fr"> '):
                 inhead = False
                 inshortdesc = False
@@ -678,7 +720,7 @@ class CacheDownloader(gobject.GObject):
             elif line.startswith('<p><span id="ctl00_ContentBody_Images">'):
                 images = line
             elif line.startswith('<span id="ctl00_ContentBody_LatLon" style="font-weight:bold;">'):
-                coords = re.compile('lat=([0-9.]+)&amp;lon=([0-9.]+)&amp;').search(line)
+                coords = re.compile('lat=([0-9.-]+)&amp;lon=([0-9.-]+)&amp;').search(line)
             elif line.startswith('<p><span id="ctl00_ContentBody_CacheLogs">'):
                 logs = line
             if inhead:
@@ -696,8 +738,8 @@ class CacheDownloader(gobject.GObject):
             if indesc:
                 desc += "%s\n" % line
                 
-            #if inhints:
-            #    hints += "%s\n" % line
+            if inhints:
+                hints += line + " "
                 
             if inwaypoints:
                 waypoints += "%s  " % line
