@@ -491,46 +491,55 @@ class HildonGui(SimpleGui):
             terrains = None
 
         if response == RESPONSE_SHOW_LIST:
-            points = self.core.get_points_filter(found=found, name_search=name_search, size=sizes, terrain=terrains, diff=difficulties, ctype=types, marked=marked)
-            self._display_results(points)
+            points, truncated = self.core.get_points_filter(found=found, name_search=name_search, size=sizes, terrain=terrains, diff=difficulties, ctype=types, marked=marked)
+            self._display_results(points, truncated)
         elif response == gtk.RESPONSE_ACCEPT:
             self.core.set_filter(found=found, name_search=name_search, size=sizes, terrain=terrains, diff=difficulties, ctype=types, marked=marked)
 
 
-    def _display_results(self, caches):
-
+    def _display_results(self, caches, truncated):
         sortfuncs = [
+            ('Dist', lambda x, y: cmp(x.prox, y.prox)),
             ('Name', lambda x, y: cmp(x.title, y.title)),
-            ('Diff.', lambda x, y: cmp(x.difficulty, y.difficulty)),
-            ('Terr.', lambda x, y: cmp(x.terrain, y.terrain)),
-            ('Size', lambda x, y: cmp(x.size, y.size)),
+            ('Diff', lambda x, y: cmp(x.difficulty if x.difficulty > 0 else 100, y.difficulty if y.difficulty > 0 else 100)),
+            ('Terr', lambda x, y: cmp(x.terrain if x.terrain > 0 else 100, y.terrain if y.terrain > 0 else 100)),
+            ('Size', lambda x, y: cmp(x.size if x.size > 0 else 100, y.size if y.size > 0 else 100)),
             ('Type', lambda x, y: cmp(x.type, y.type)),
         ]
 
-
+        if self.gps_data != None and self.gps_data.position != None:
+            for c in caches:
+                c.prox = c.distance_to(self.gps_data.position)
+                print c.difficulty
+        else:
+            for c in caches:
+                c.prox = None
 
         win = hildon.StackableWindow()
         win.set_title("Search results")
-
-        ls = gtk.ListStore(str, str, str, object)
+        ls = gtk.ListStore(str, str, str, str, object)
         
         tv = hildon.TouchSelector()
         col1 = tv.append_column(ls, gtk.CellRendererText())
         
         c1cr = gtk.CellRendererText()
+        c1cr.ellipsize=pango.ELLIPSIZE_MIDDLE
         c2cr = gtk.CellRendererText()
         c3cr = gtk.CellRendererText()
+        c4cr = gtk.CellRendererText()
         
         col1.pack_start(c1cr, True)
-        col1.pack_end(c2cr, True)
-        col1.pack_start(c3cr, True)
+        col1.pack_end(c2cr, False)
+        col1.pack_start(c3cr, False)
+        col1.pack_end(c4cr, False)
 
         col1.set_attributes(c1cr, text=0)
         col1.set_attributes(c2cr, text=1)
         col1.set_attributes(c3cr, text=2)
+        col1.set_attributes(c4cr, text=3)
 
         def select_cache(widget, data, more):
-            self.show_cache(self._get_selected(widget)[3])
+            self.show_cache(self._get_selected(widget)[4])
         
         tv.connect("changed", select_cache, None)
 
@@ -540,7 +549,7 @@ class HildonGui(SimpleGui):
             ls.clear()
             caches.sort(cmp=sortfunc)
             for c in caches:
-                ls.append([c.title, c.get_size_string(), 'D%s T%s' % (c.get_difficulty(), c.get_terrain()), c])
+                ls.append([self.shorten_name(c.title, 40), " " + c.get_size_string(), ' D%s T%s' % (c.get_difficulty(), c.get_terrain()), " " + self.__format_distance(c.prox),c])
             tv.handler_unblock_by_func(select_cache)
 
 
@@ -559,6 +568,8 @@ class HildonGui(SimpleGui):
         on_change_sort(None, sortfuncs[0][1])
 
         win.show_all()
+        if truncated:
+            hildon.hildon_banner_show_information_with_markup(win, "hu", "Showing only the first %d results." % len(caches))
         self.old_search_window = win
         self.reopen_last_search_button.set_sensitive(True)
 
@@ -664,13 +675,13 @@ class HildonGui(SimpleGui):
         if self.current_cache != None and self.current_cache.name == cache.name and self.old_cache_window != None:
             
             hildon.WindowStack.get_default().push_1(self.old_cache_window)
-
             self.current_cache_window_open = True
             return
         
         if select:
             self.set_current_cache(cache)
 
+        
         win = hildon.StackableWindow()
         win.set_title(cache.title)
         
@@ -716,7 +727,7 @@ class HildonGui(SimpleGui):
             p.attach(gtk.Label("Please download full details to see the description."), 0, 2, 9, 10)
         
         notebook.append_page(p, gtk.Label("info"))
-
+        self.cache_calc = None
         if cache.was_downloaded():
         
             # desc
@@ -781,46 +792,43 @@ class HildonGui(SimpleGui):
                 self.build_cache_images(cache, notebook)
 
             # calculated coords
-            self.cache_calc_vars = {}
-            self.cache_calc = coordfinder.CalcCoordinate.find(text_longdesc)
-            if len(self.cache_calc[0]) > 0:
+            text = text_longdesc
+            for w in cache.get_waypoints():
+                text += " | " + w['comment']
+            self.cache_calc = coordfinder.CalcCoordinateManager(cache, text)
+            if len(self.cache_calc.coords) > 0:
                 self.build_cache_calc(cache, notebook)
 
         # coords
 
 
         p = gtk.VBox()
-        widget_coords, clist = self._get_coord_selector(cache, lambda x, y, z: True)
-
-
-        def set_coord_as_target(widget):
-            tm = widget_coords.get_model(0)
-            iter = tm.get_iter(0)
-            widget_coords.get_selected(0, iter)
-            c = clist[tm.get_path(iter)[0]]
-            if c == None:
+        #self.build_coordinates(cache, p)
+        self.cache_coord_page = p
+        x = notebook.get_n_pages()
+        notebook.append_page(p, gtk.Label("coords"))
+        def switchpage(caller, page, pageno):
+            if pageno != x:
                 return
-            self.set_current_cache(cache)
-            self.set_target(c)
-            self.hide_cache_view()
+            cache.notes = self.get_cache_notes()
+            self.update_coords()
+        notebook.connect("switch-page", switchpage)
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-        button.set_label("set as target")
-        button.connect("clicked", set_coord_as_target)
-        
-        p.pack_start(widget_coords, True, True)
-        p.pack_start(button, False, True)
-        notebook.append_page(p, gtk.Label("coords"))      
-        
         # notes
         p = hildon.PannableArea()
         self.cache_notes = gtk.TextView()
         #cache_notes.set_editable(True)
         self.cache_notes.get_buffer().set_text(cache.notes)
         p.add(self.cache_notes)
-        
+
         notebook.append_page(p, gtk.Label("notes"))
-        
+        self.notes_changed = False
+        def notes_changed(caller):
+            self.notes_changed = True
+
+        self.cache_notes.get_buffer().connect('changed', notes_changed)
+
+
         win.add(notebook)
         
         # menu
@@ -863,6 +871,53 @@ class HildonGui(SimpleGui):
         win.connect('delete_event', self.hide_cache_view)
         self.current_cache_window_open = True
 
+    def build_coordinates(self, cache, p):
+        print 'updating'
+        widget_coords, clist = self._get_coord_selector(cache, lambda x, y, z: True)
+
+
+        def set_coord_as_target(widget):
+            tm = widget_coords.get_model(0)
+            iter = tm.get_iter(0)
+            widget_coords.get_selected(0, iter)
+            c = clist[tm.get_path(iter)[0]]
+            if c == None:
+                return
+            self.set_current_cache(cache)
+            self.set_target(c)
+            self.hide_cache_view()
+
+        def set_alternative_position(widget):
+            tm = widget_coords.get_model(0)
+            iter = tm.get_iter(0)
+            widget_coords.get_selected(0, iter)
+            c = clist[tm.get_path(iter)[0]]
+            if c == None:
+                return
+            self.core.set_alternative_position(cache, c)
+            
+        h = gtk.HBox()
+        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        button.set_label("set as target")
+        button.connect("clicked", set_coord_as_target)
+        h.pack_start(button)
+
+        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        button.set_label("set as main coordinate")
+        button.connect("clicked", set_alternative_position)
+        h.pack_start(button)
+        
+        p.pack_start(widget_coords, True, True)
+        p.pack_start(h, False, True)
+        widget_coords.show_all()
+        h.show_all()
+
+    def update_coords(self):
+        for x in self.cache_coord_page.get_children():
+            self.cache_coord_page.remove(x)
+        self.build_coordinates(self.current_cache, self.cache_coord_page)
+        self.cache_coord_page.show()
+
     def build_cache_images(self, cache, notebook):
         selector = hildon.TouchSelector(text=True)
         selector.get_column(0).get_cells()[0].set_property('xalign', 0)
@@ -894,24 +949,17 @@ class HildonGui(SimpleGui):
     def build_cache_calc(self, cache, notebook):
 
         def input_changed(widget, char):
-            value = widget.get_text()
-            if value != '':
-                self.cache_calc_vars[char] = widget.get_text()
-            else:
-                del self.cache_calc_vars[char]
+            self.cache_calc.set_var(char, widget.get_text())
             self.show_cache_calc_results()
 
-        
-        coords, requires = self.cache_calc
-        self.cache_calc_vars = cache.get_vars()
         p = gtk.VBox()
-        count = len(requires)
+        count = len(self.cache_calc.requires)
         # create table with n columns.
         cols = 7
         rows = int(math.ceil(float(count) / float(cols)))
         table = gtk.Table(rows, cols)
         i = 0
-        requires_sort = list(requires)
+        requires_sort = list(self.cache_calc.requires)
         requires_sort.sort()
         for char in requires_sort:
             row = i / cols
@@ -920,8 +968,8 @@ class HildonGui(SimpleGui):
             m.pack_start(gtk.Label(str(char)))
             e = hildon.Entry(gtk.HILDON_SIZE_AUTO)
             e.set_property("hildon-input-mode", gtk.HILDON_GTK_INPUT_MODE_NUMERIC)
-            if char in self.cache_calc_vars.keys():
-                e.set_text(self.cache_calc_vars[char])
+            if char in self.cache_calc.vars.keys():
+                e.set_text(self.cache_calc.vars[char])
             e.connect('changed', input_changed, str(char))
             e.set_size_request(50, -1)
             m.pack_start(e)
@@ -940,25 +988,19 @@ class HildonGui(SimpleGui):
     def show_cache_calc_results(self):
         p = gtk.VBox()
         
-        for c in self.cache_calc[0]:
-            c.set_vars(self.cache_calc_vars)
+        for c in self.cache_calc.coords:
             label_text = '<b>%s</b>\n' % c.orig
             button = None
             if c.has_requires():
-                result = c.try_get_solution()
                 label_text += '= %s\n' % c.replaced_result
-                if result != False:
-                    label_text += '= %s\n' % result
+                if c.result != False:
+                    label_text += '= %s\n' % c.result
                 for warning in c.warnings:
                     label_text += "<b>!</b> <span color='gold'>%s</span>\n" % warning
-                if result != False:
-                    button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-                    button.set_label("use")
-                    button.connect('clicked', self._on_set_target_clicked, result)
             else:
                 label_text += "<i>Needs "
                 for r in c.requires:
-                    if r in self.cache_calc_vars.keys():
+                    if r in self.cache_calc.vars.keys():
                         label_text += "<s>%s </s>" % r
                     else:
                         label_text += "<b>%s </b>" % r
@@ -997,11 +1039,26 @@ class HildonGui(SimpleGui):
         vp.add(i)
         win.show_all()
 
+    @staticmethod
+    def wrap(text, width):
+        """
+        A word-wrap function that preserves existing line breaks
+        and most spaces in the text. Expects that existing line
+        breaks are posix newlines (\n).
+        """
+        return reduce(lambda line, word, width=width: '%s%s%s' %
+                  (line,
+                   ' \n'[(len(line)-line.rfind('\n')-1
+                         + len(word.split('\n',1)[0]
+                              ) >= width)],
+                   word),
+                  text.split(' ')
+                 )
+
     def _get_coord_selector(self, cache, callback, no_empty=False):
         selector = hildon.TouchSelector(text=True)
         selector.get_column(0).get_cells()[0].set_property('xalign', 0)
         selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
-
         format = lambda n: "%s %s" % (re.sub(r' ', '', n.get_lat(self.format)), re.sub(r' ', '', n.get_lon(self.format)))
         selector.append_text("First Waypoint: %s" % format(cache))
         clist = {0: cache}
@@ -1015,9 +1072,23 @@ class HildonGui(SimpleGui):
             else:
                 coord = None
                 latlon = '???'
-            selector.append_text("%s - %s - %s\n%s" % (w['name'], latlon, w['id'], self._strip_html(w['comment'])))
+            text = self.wrap("%s - %s - %s\n%s" % (w['name'], latlon, w['id'], self._strip_html(w['comment'])), 80)
+            selector.append_text(text)
             clist[i] = coord
             i += 1
+        
+        for coord in geo.search_coordinates(cache.notes):
+            selector.append_text("manually entered: " + format(coord))
+            clist[i] = coord
+            i += 1
+        if self.cache_calc != None:
+            for coord in self.cache_calc.get_solutions():
+                selector.append_text("calculated: %s = %s" % (coord.name, format(coord)))
+                clist[i] = coord
+                i += 1
+
+        
+
         selector.connect('changed', callback, clist)
         return selector, clist
 
@@ -1282,14 +1353,20 @@ class HildonGui(SimpleGui):
 
 
     def hide_cache_view(self, widget=None, data=None):
-        self.current_cache.set_vars(self.cache_calc_vars)
-        self.core.set_cache_calc_vars(self.current_cache, self.current_cache.vars)
+        if self.cache_calc != None:
+            self.current_cache.set_vars(self.cache_calc.vars)
+            self.core.set_cache_calc_vars(self.current_cache, self.current_cache.vars)
         self.current_cache_window_open = False
-        b = self.cache_notes.get_buffer()
-        self.core.on_notes_changed(self.current_cache, b.get_text(b.get_start_iter(), b.get_end_iter()))
+        if self.notes_changed:
+            self.core.on_notes_changed(self.current_cache, self.get_cache_notes())
+            self.notes_changed = False
         self.old_cache_window = hildon.WindowStack.get_default().peek()
         hildon.WindowStack.get_default().pop(hildon.WindowStack.get_default().size()-1)
         return True
+
+    def get_cache_notes(self):
+        b = self.cache_notes.get_buffer()
+        return b.get_text(b.get_start_iter(), b.get_end_iter())
                 
 
     #called by core
@@ -1336,7 +1413,7 @@ class HildonGui(SimpleGui):
             text = "No sats, error: ±%3.1fm" % self.gps_data.error
         else:
             text = "%d/%d sats, error: ±%3.1fm" % (self.gps_data.sats, self.gps_data.sats_known, self.gps_data.error)
-        self.label_quality.set_markup("Accurancy\n<small>%s</small>" % text)
+        self.label_quality.set_markup("Accuracy\n<small>%s</small>" % text)
         if self.gps_data.altitude == None or self.gps_data.bearing == None:
             return
 
@@ -1348,13 +1425,19 @@ class HildonGui(SimpleGui):
             return
 
         target_distance = self.gps_data.position.distance_to(self.current_target)
-        if target_distance >= 1000:
-            label = "%d km" % round(target_distance / 1000)
-        elif target_distance >= 100:
-            label = "%d m" % round(target_distance)
+        
+        self.label_dist.set_markup("Distance\n<small>%s</small>" % self.__format_distance(target_distance))
+
+    def __format_distance(self, distance):
+        if distance == None:
+            return '?'
+        if distance >= 1000:
+            return "%d km" % round(distance / 1000)
+        elif distance >= 100:
+            return "%d m" % round(distance)
         else:
-            label = "%.1f m" % round(target_distance, 1)
-        self.label_dist.set_markup("Distance\n<small>%s</small>" % label)
+            return "%.1f m" % round(distance, 1)
+
 
 
     def on_no_fix(self, gps_data, status):
@@ -1412,4 +1495,4 @@ class HildonGui(SimpleGui):
         # /Reading & Writing Settings
         #
         ##############################################
-        
\ No newline at end of file
+        
